/* -*- c-basic-offset: 2 -*- */

package nwnintf;
import java.io.*;
import nwnintf.format.mod.*;
import nwnintf.format.itp.*;
import nwnintf.format.itp.element.*;

/**
 * Refer to a script slot of a blueprint.
 */

public class BlueSlotRef implements ScriptSlotRef {

  ResRef blueprint_;
  String slot_;

  public BlueSlotRef(ResRef blueprint, String slot) throws Exception {
      
    this( blueprint,slot, true);
  }

  /**
   * @param translate If true, translate name of script slot to
   * internal .mod file format. otherwise do not.
   */
  public BlueSlotRef(ResRef blueprint, String slot, boolean translate)
    throws Exception {
    blueprint_ = blueprint;
    if( translate ) slot_ = translateScriptSlot(blueprint, slot);
    else slot_ = slot;
  }

  public ResRef getResref() {
    return blueprint_;
  }

  public String getSlot() {
    return slot_;
  }

  public SavedSlot save(ModFile mod, SezFile sez, ResRef script) 
    throws Exception {
    return SavedBlueSlot.saveNew(this,mod,sez,script);
  }


  private static String translateScriptSlot(ResRef resref, String slot)
    throws Exception
  {
    if( resref.typeInt() == ResRef.tiUTC ) {
      // Creature
      if( slot.equals("OnHeartbeat") ) return "ScriptHeartbeat";
      if( slot.equals("OnPerception") ) return "ScriptOnNotice";
      if( slot.equals("OnSpellAt") ) return "ScriptSpellAt";
      if( slot.equals("OnPhysicalAttacked") ) return "ScriptAttacked";
      if( slot.equals("OnDamaged") ) return "ScriptDamaged";
      if( slot.equals("OnDisturbed") ) return "ScriptDisturbed";
      if( slot.equals("OnCombatRoundEnd") ) return "ScriptEndRound";
      if( slot.equals("OnConversation") ) return "ScriptDialogue";
      if( slot.equals("OnSpawn") ) return "ScriptSpawn";
      if( slot.equals("OnRested") ) return "ScriptRested";
      if( slot.equals("OnDeath") ) return "ScriptDeath";
      if( slot.equals("OnUserDefined") ) return "ScriptUserDefine";
      if( slot.equals("OnBlocked") ) return "ScriptOnBlocked";
    }
    if( resref.typeInt() == ResRef.tiUTD ) {
      // Door
      if( slot.equals("OnClose") ) return " OnClosed";
      if( slot.equals("OnDamaged") ) return " OnDamaged";
      if( slot.equals("OnDeath") ) return "OnDeath";
      if( slot.equals("OnDisarm") ) return "OnDisarm";
      if( slot.equals("OnHeartbeat") ) return "OnHeartbeat";
      if( slot.equals("OnLock") ) return "OnLock";
      if( slot.equals("OnPhysicalAttacked") ) return "OnMeleeAttacked";
      if( slot.equals("OnOpen") ) return "OnOpen";
      if( slot.equals("OnSpellCastAt") ) return "OnSpellCastAt";
      if( slot.equals("OnTrapTriggered") ) return "OnTrapTriggered";
      if( slot.equals("OnUnLock") ) return "OnUnlock";
      if( slot.equals("OnUserDefined") ) return "OnUserDefined";
      if( slot.equals("OnAreaTransitionClick") ) return "OnClick";
      if( slot.equals("OnFailToOpen") ) return "OnFailToOpen";
    }
    if( resref.typeInt() == ResRef.tiUTE ) {
      // Encounter
      if( slot.equals("OnEnter") ) return "OnEntered";
      if( slot.equals("OnExhausted") ) return "OnExhausted";
      if( slot.equals("OnExit") ) return "OnExit";
      if( slot.equals("OnHeartbeat") ) return "OnHeartbeat";
      if( slot.equals("OnUserDefined") ) return "OnUserDefined";
    }
    if( resref.typeInt() == ResRef.tiUTI ) {
      // Item
    }
    if( resref.typeInt() == ResRef.tiUTM ) {
      // Store
      if( slot.equals("OnOpenStore") ) return "OnOpenStore";
    }
    if( resref.typeInt() == ResRef.tiUTP ) {
      // Placeable
      if( slot.equals("OnClose") ) return "OnClosed";
      if( slot.equals("OnDamaged") ) return "OnDamaged";
      if( slot.equals("OnDeath") ) return "OnDeath";
      if( slot.equals("OnHeartbeat") ) return "OnHeartbeat";
      if( slot.equals("OnDisturbed") ) return "OnInvDisturbed";
      if( slot.equals("OnLock") ) return "OnLock";
      if( slot.equals("OnPhysicalAttacked") ) return "OnMeleeAttacked";
      if( slot.equals("OnOpen") ) return "OnOpen";
      if( slot.equals("OnSpellCastAt") ) return "OnSpellCastAt";
      if( slot.equals("OnUnLock") ) return "OnUnlock";
      if( slot.equals("OnUsed") ) return "OnUsed";
      if( slot.equals("OnUserDefined") ) return "OnUserDefined";
      if( slot.equals("OnDisarm") ) return "OnDisarm";
      if( slot.equals("OnTrapTriggered") ) return "OnTrapTriggered";
    }
    if( resref.typeInt() == ResRef.tiUTS ) {
      // Sound
    }
    if( resref.typeInt() == ResRef.tiUTT ) {
      // Trigger
      if( slot.equals("OnClick") ) return "OnClick";
      if( slot.equals("OnEnter") ) return "ScriptOnEnter";
      if( slot.equals("OnExit") ) return "ScriptOnExit";
      if( slot.equals("OnHeartbeat") ) return "ScriptHeartbeat";
      if( slot.equals("OnUserDefined") ) return "ScriptUserDefine";
      if( slot.equals("OnDisarm") ) return "OnDisarm";
      if( slot.equals("OnTrapTriggered") ) return "OnTrapTriggered";
    }
    if( resref.typeInt() == ResRef.tiUTW ) {
      // Waypoint
    }

    throw new Exception("Unrecognized slot: ("+resref.filename()+")("+slot+")");
      
       
  }

  public boolean equals(Object o) {
    if( o == null ) return false;
    if( o instanceof BlueSlotRef ) {
      BlueSlotRef r = (BlueSlotRef)o;
      return
	r.slot_.equals(this.slot_) &&
	r.blueprint_.equals(this.blueprint_);
    }
    return false;
  }

  public int hashCode() {
    /* invert type so that higher-order bits of the hash code tend to be
     * very different when names are the same. I'm guessing that will
     * improve performance on hashtables since most type codes are different
     * only in the bottom few bits.
     */
    return blueprint_.hashCode() ^ slot_.hashCode();
  }

  public String toString() {
    return blueprint_.toString() + ":" + slot_;
  }
}
