/* -*- c-basic-offset: 2 -*- */

package nwnintf;

import java.io.*;

/**
 * Quick and dirty parser for windows-format .ini files.
 * java.util.prefs.Preferences lets us store keys in the registry,
 * but only in a sandbox that doesn't let us grab keys from anywhere
 * we want.
 * Everything goes under either
 * \\HKLM\SOFTWARE\JavaSoft\Prefs
 * \\HKCU\Software\JavaSoft\Prefs
 */
public class IniParser {
  File f_;
  
  /**
   * @param f The .ini file.
   */
  public IniParser(File f) {
    f_ = f;
  }

  /**
   * Retrieve a key from the section of the inifile.
   * @param section Name of the section to look for the key in.
   * @param key Name of the key.
   * @return value of the key. Null if the section of key did not exist.
   */
  public String get(String section, String key) throws IOException {
    BufferedReader br = new BufferedReader( new FileReader(f_) );

    String line;
    do {

      line = br.readLine();

      if( line != null && line.equals("["+section+"]") ) {
	// Found section.

	do {
	  line = br.readLine();

	  if( line != null && line.startsWith( key + "=" ) ) {
	    // Found key.

	    int idx = line.indexOf('=');
	    String value = line.substring(idx + 1);
	    return value;

	  }
	} while( line != null );
      }
    } while( line != null );

    return null;
  }

  /**
   * command-line test
   */
  public static void main(String[] args) throws Exception {
    String file = args[0];
    String section = args[1];
    String key = args[2];

    System.out.println(new IniParser( new File(file) ).get(section,key));
  }
    
}
