/* -*- c-basic-offset: 2 -*- */

package nwnintf;
import java.io.*;
import java.util.*;

import nwnintf.format.mod.*;
import nwnintf.format.itp.*;
import nwnintf.format.itp.element.*;

public class InstanceUpdater {

  NwnModule nwnmodule_;

  /**
   * Map< ResRef, Collection<SlotValue> >
   */
  Map slotmap_; 

  /**
   * @param slotvalues a Collection of SlotValue instances summarizing
   * the changes that have been made to blueprints.
   */
  public InstanceUpdater(NwnModule nwnmodule, Collection slotvalues) {
    nwnmodule_ = nwnmodule;

    slotmap_ = buildResMap(slotvalues);
  }

  /**
   * Map resrefs to the relevant SlotValue instances, for quick lookup
   */
  private static Map buildResMap(Collection slotvalues) {
    Map m = new HashMap();
    for( Iterator it = slotvalues.iterator(); it.hasNext(); ) {
      SlotValue slot = (SlotValue)it.next();

      //System.err.println(slot);

      /* We only know how to handle blueprints.
       */
      if( slot.getSlotRef() instanceof BlueSlotRef ) {

	ResRef ref = ((BlueSlotRef)slot.getSlotRef()).getResref();
	/* See if we have already have other slots in that resref here.
	 * We're maintaining a list of all slotrefs for a given resref.
	 */
	Object val = m.get( ref );
	Collection coll;
	if( val == null ) { coll = new Vector(); m.put(ref,coll); }
	else { coll = (Collection)val; }
	//System.err.println("added " + slot + " to collection for " + ref);
	coll.add( slot );
      }
      
    }
    return m;

  }

  public void updateAllAreas( ) throws Exception {
    /* Iterate over areas.
     */
    for( Iterator it = nwnmodule_.getModfile().iterator( ResRef.tiGIT );
	 it.hasNext();
	 ) {
      ResRef area = (ResRef)it.next();
      System.err.println("Updating area " + area);
      updateOneArea(area);
    }

  }
  /* Returns true iff it made any changes in the area.
   */
  public boolean updateOneArea( ResRef areares ) throws Exception {

    boolean result = false;

    ItpFile area = new ItpFile( nwnmodule_.getModfile().resource( areares ) );

    // Iterate over lists of placed instances in this area.
    for( Iterator keys = instlisttypes_.keySet().iterator();
	 keys.hasNext();
	 ) {
      String key = (String)keys.next();
      ItpElement instlist = area.root().element( key );
      if( instlist == null ) {
	System.err.println("no inst key " + key + " in area " + areares);
	continue;
      }
      ItpListElt list = (ItpListElt)instlist;
      //System.err.println("There are " + list.length() + " instances in " + key);
      // Iterate over placed instances of whatever type of thing it is.
      for( int i = 0; i < list.length(); ++i ) {
	if( updateOneInstance( (String)instlisttypes_.get(key),
			       list.entry(i) ) ) {
	  result = true;
	}
      }
    }
    if( result ) {
      /* Some instance was updated, so re-write the area back to the
       * module file.
       */
      nwnmodule_.getModfile().addupdateresource(areares,
						new ItpWriter( area ).bytes());
    }

    return result;
  }

  /* Returns true iff it made a change to the instance.
   */
  public boolean updateOneInstance( String type, ItpEntry inst ) throws Exception {
    boolean result = false;
    ItpResrefElt resref = (ItpResrefElt)inst.element( templateresrefKey );
    if( resref == null ) {
      System.err.println("no template key " + templateresrefKey +
			 " in instance of type " + type);
    }
    //System.err.println("an instance with template " + resref.value());
    Object oslots = slotmap_.get( new ResRef( resref.value(), type ) );
    if( oslots != null ) {
      //System.err.println("has slots");
      /* There are slots to be updated in this instance.
       */
      Collection slots = (Collection)oslots;
      for( Iterator it = slots.iterator(); it.hasNext(); ) {
	SlotValue slotvalue = (SlotValue)it.next();
	/* By construction of slotmap_, these must be BlueSlotRef.
	 */
	BlueSlotRef slotref = (BlueSlotRef)slotvalue.getSlotRef();

	/* Get the script slot in the instance...
	 */
	ItpResrefElt slotelt = 
	  (ItpResrefElt)inst.element( slotref.getSlot() );

	if( slotelt == null ) {
	  System.err.println("missing slot " + slotref.getSlot() +
			     " of an instance");
	  continue;
	}
	/* ... and set it to whatever the blueprint has in that
	 * slot. (using the basename of the resref only because the
	 * type is implicit to the slot, and is not explicity stored)
	 */
	//System.err.println("Updating some instance " + type + " " + slotref.getSlot());
	slotelt.setValue( slotvalue.getValue().getBasename() );
	result = true;
      }
    }
    return result;
  }

  public static String creatureInstances = "Creature List";
  public static String doorInstances = "Door List";
  public static String encounterInstances = "Encounter List";
  public static String soundInstances = "SoundList";
  public static String storeInstances = "StoreList";
  public static String triggerInstances = "TriggerList";
  public static String waypointInstances = "WaypointList";
  public static String placeableInstances = "Placeable List";

  public static String templateresrefKey = "TemplateResRef";

  private static Map instlisttypes_;
  static {
    instlisttypes_ = new HashMap();
    instlisttypes_.put( creatureInstances, ResRef.tsUTC );
    instlisttypes_.put( doorInstances, ResRef.tsUTD );
    instlisttypes_.put( encounterInstances, ResRef.tsUTE );
    instlisttypes_.put( soundInstances, ResRef.tsUTS );
    instlisttypes_.put( storeInstances, ResRef.tsUTM );
    instlisttypes_.put( triggerInstances, ResRef.tsUTT );
    instlisttypes_.put( waypointInstances, ResRef.tsUTW );
    instlisttypes_.put( placeableInstances, ResRef.tsUTP );
  }



}
