/* -*- c-basic-offset: 4 -*- */
package nwnintf;
import nwnintf.format.mod.*;
import nwnintf.format.tlk.*;
import nwnintf.format.itp.*;
import nwnintf.format.itp.element.*;
import nwnintf.palette.*;

/**
 * Encapsulate the visible properties of a custom blueprint in a Module.
 * Accessor methods return things as string rather than internal scriptease
 * types to avoid dependencies in nwnintf on the scriptease package
 * hierarchy.
 */
public class ModuleBlueprint {
    ResRef resref_;
    // Only cached between invocations of tag(), not stored permanently.
    NwnModule module_;

    String name_;
    String tag_;


    /**
     * @param nwnmodule The module the blueprint is in.
     * @param resref The resref the blueprint.
     * @param name A display name for this blueprint.
     */
    public ModuleBlueprint( ResRef resref, String name) {

      resref_ = resref;
      name_ = name;

    }

    public ModuleBlueprint() {
    }
    public String getName() {
	return name_;
    }
    public void setName(String name) {
	name_ = name;
    }
    public ResRef getResRef() {
	return resref_;
    }
    public void setResRef(ResRef resref) {
	resref_ = resref;
    }


    /**
     * Tag of this entity.
     */
    public String tag(NwnModule nwnmodule) {
      // Lazy.
      if( nwnmodule == module_  && tag_ != null ) return tag_;
      module_ = nwnmodule;

      try {
	  ItpFile itp = new ItpFile(nwnmodule.getModfile().resource(resref()));
	  tag_ = ((ItpStringElt)itp.root().element("Tag")).value();
      } catch(Exception e) {
      }
      if( tag_ == null ) return "";
      return tag_;
    }


    /**
     * A display name for this blueprint, such as would appear in the
     * blueprint tree display in Aurora.
     */
    public String name()  {
      return name_;

      // name_ = ((ItpStringElt)entry_.element("NAME")).value();
    }

    /**
     * The ResRef of this blueprint.
     */
    public ResRef resref() {
      return resref_;
    }

    /**
     * Equals method for ModuleBlueprints.  ModuleBlueprints are equal if
     * their resrefs are equal
     */
    public boolean equals(Object obj) {
      if( obj instanceof ModuleBlueprint ) {
        ModuleBlueprint bp = (ModuleBlueprint) obj;
        return this.resref_.equals(bp.resref_);
      }
      return false;
    }

    /**
     * Hash Code for this blueprint.
     */
    public int hashCode() {
      return this.resref_.hashCode();
    }

}

