/* -*- c-basic-offset: 2 -*- */

package nwnintf;
import java.io.*;
import java.util.*;

import nwnintf.format.mod.ModFile;
import nwnintf.format.mod.ResRef;


/** Compile scripts and attach them to a module file.
 */

public class ScriptInstaller {

  private File compiledir_;

  private File nwnnsscomp_;

  private NwnModule nwnmodule_;

  /*
      String scriptname = "scriptease" + Integer.toString(i);
      try {
	si.installScript( scriptname, eCode.generateCode());
      } catch(Exception e) {
	System.err.println("Failed compile");
	e.printStackTrace();
	continue;
      }
      System.out.println("Attaching to \""+eCode.getResref()+"\" slot \""+
			 eCode.getEventName() + "\" script " + scriptname);
      si.attachScript( eCode.getResref(), eCode.getEventName(), scriptname );

    }
  }
  */


  public ScriptInstaller(NwnModule nwnmodule, HashMap includeFiles) throws Exception {

    nwnmodule_ = nwnmodule;

    String tmpdirname = System.getProperty("java.io.tmpdir");
    File f = new File(tmpdirname);
    compiledir_ = new File( f, "scriptease-compile" );
    compiledir_.mkdirs();

    nwnnsscomp_ = new File( new File( new File("."),
				      "nwnnsscomp" ),
			    "nwnnsscomp.exe" );

    this.addIncludeFiles(includeFiles);

/*
    this.copyFile( new File(".", "atoms.txt"),
		   new File(compiledir_, "i_se_atoms.nss") );

    String atomssrc = readFileAsString(new File(".", "atoms.txt"));

    //String atomsbytecode = compileFile( atomssrc );
    installScript( "i_se_atoms", atomssrc );
*/
  }

  public void addIncludeFiles(HashMap includeFiles) throws Exception {
    Iterator iter;
    String name, code;
    Map.Entry entry;

    iter = includeFiles.entrySet().iterator();
    while( iter.hasNext() ) {
      entry = (Map.Entry) iter.next();
      name = (String) entry.getKey();
      code = (String) entry.getValue();

      writeStringToFile(code, new File(compiledir_, name + ".nss") );
      installScript( name, code );
    }

  }

  /** 
   * Compile a neverwinter script file. The file is only allowed to
   * include "i_se_atoms". The return value will be the compile bytecode
   * (.ncs format).
   * Throws Exception if there was a compile error.
   * @param contents Script file text
   * @return bytecode string.
   */

  public byte[] compileFile(String contents) throws Exception {
    File scriptfile = File.createTempFile("scez", ".nss", compiledir_);
    File bytecodefile =File.createTempFile("scez", ".ncs", compiledir_);

    this.writeStringToFile( contents, scriptfile );

    String[] cmdarray = new String[3];
    cmdarray[0] = nwnnsscomp_.getAbsolutePath();
    cmdarray[1] = scriptfile.getAbsolutePath();
    cmdarray[2] = bytecodefile.getAbsolutePath();

    Process p = Runtime.getRuntime().exec( cmdarray, null, compiledir_ );
    int exitstatus;
    try {
      InputStream is = p.getInputStream();
      int x;
      while(true) { x = is.read(); if( x==-1) break; System.out.print((char)x); }
      exitstatus = p.waitFor();
    } catch(Exception e) {
      e.printStackTrace();
      exitstatus = 1;
    }

    byte[] result = readFileAsBytes(bytecodefile);
    scriptfile.deleteOnExit();
    bytecodefile.deleteOnExit();

    if( exitstatus != 0 ) {
      throw new Exception("Compile errors");
    }
      
    return result;

  }


  /** Install a script into the module using the given resource name.
   * Don't put .nss or anything on the resname.
   * @param resname Name to use for this resource. If a script resource already
   * exists by this name, it will be overwritten.
   * @param contents String containing the script text.
   */
  public void installScript( String resname, String contents )
    throws Exception {

    byte[] bytecode = this.compileFile(contents);

    ModFile mod = nwnmodule_.getModfile();
    mod.addupdateresource( new ResRef(resname, ResRef.tiNSS),
			   contents.getBytes() );
    mod.addupdateresource( new ResRef(resname, ResRef.tiNCS), bytecode );
  }

  /**
   * @param scriptslots Contains ScriptSlot instances.
   * @see attachScript( String, String, String )
   * @see compileFile( String )
   */
  public void installScripts( Collection scriptslots ) throws Exception {
    for( Iterator it = scriptslots.iterator();
	 it.hasNext(); ) {
      ScriptSlot ss = (ScriptSlot)it.next();
      byte[] bytecode = compileFile( ss.getSourcecode() );
      ss.setBytecode( bytecode );
    }
    /* All files have successfully compiled. Update the blueprints and
     * conversations.
     */
    Collection slotvalues = nwnmodule_.getSavedScripts().save( scriptslots );

    System.err.println("saved blueprints...");

    /* Now update blueprint instances.
     */
    InstanceUpdater instupdater = new InstanceUpdater(nwnmodule_, slotvalues);
    System.err.println("instantiated InstanceUpdater");
    instupdater.updateAllAreas();

  }


  private static void copyFile(File src, File dst) throws IOException {
    InputStream is = new FileInputStream(src);
    OutputStream os = new FileOutputStream(dst);

    is = new BufferedInputStream(is);
    os = new BufferedOutputStream(os);

    int x;
    while(true) {
      x = is.read();
      if( x == -1 ) break;
      os.write(x);
    }
    os.close();
    is.close();
  }

  private static void writeStringToFile(String src, File dst)
    throws IOException {
    OutputStream os = new FileOutputStream(dst);
    os = new BufferedOutputStream(os);

    for(int i = 0; i < src.length(); ++i ) {
      os.write( src.charAt(i) );
    }
    os.close();
  }

  private static String readFileAsString(File src) throws IOException {
    InputStream is = new FileInputStream(src);
    is = new BufferedInputStream(is);
    
    /* There's probably a better way to do this, but to hell with it.
     * when in java land.
     */
    StringBuffer buf = new StringBuffer();
    int x;
    while(true) {
      x = is.read();
      if( x == -1 ) break;
      buf.append( (char)x );
    }
    is.close();
    return buf.toString();
  }

  private static byte[] readFileAsBytes(File src) throws IOException {
    InputStream is = new FileInputStream(src);
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    int x;
    while(true) {
      x = is.read();
      if( x == - 1 ) break;
      baos.write(x);
    }
    is.close();
    return baos.toByteArray();
  }

  private static String translateScriptSlot(String resname, String slot)
  throws Exception
  {
    if( resname.endsWith("utc") ) {
      // Creature
      if( slot.equals("OnHeartbeat") ) return "ScriptHeartbeat";
      if( slot.equals("OnPerception") ) return "ScriptOnNotice";
      if( slot.equals("OnSpellAt") ) return "ScriptSpellAt";
      if( slot.equals("OnPhysicalAttacked") ) return "ScriptAttacked";
      if( slot.equals("OnDamaged") ) return "ScriptDamaged";
      if( slot.equals("OnDisturbed") ) return "ScriptDisturbed";
      if( slot.equals("OnCombatRoundEnd") ) return "ScriptEndRound";
      if( slot.equals("OnConversation") ) return "ScriptDialogue";
      if( slot.equals("OnSpawn") ) return "ScriptSpawn";
      if( slot.equals("OnRested") ) return "ScriptRested";
      if( slot.equals("OnDeath") ) return "ScriptDeath";
      if( slot.equals("OnUserDefined") ) return "ScriptUserDefine";
      if( slot.equals("OnBlocked") ) return "ScriptOnBlocked";
    }
    if( resname.endsWith("utd") ) {
      // Door
      if( slot.equals("OnClose") ) return " OnClosed";
      if( slot.equals("OnDamaged") ) return " OnDamaged";
      if( slot.equals("OnDeath") ) return "OnDeath";
      if( slot.equals("OnDisarm") ) return "OnDisarm";
      if( slot.equals("OnHeartbeat") ) return "OnHeartbeat";
      if( slot.equals("OnLock") ) return "OnLock";
      if( slot.equals("OnPhysicalAttacked") ) return "OnMeleeAttacked";
      if( slot.equals("OnOpen") ) return "OnOpen";
      if( slot.equals("OnSpellCastAt") ) return "OnSpellCastAt";
      if( slot.equals("OnTrapTriggered") ) return "OnTrapTriggered";
      if( slot.equals("OnUnLock") ) return "OnUnlock";
      if( slot.equals("OnUserDefined") ) return "OnUserDefined";
      if( slot.equals("OnAreaTransitionClick") ) return "OnClick";
      if( slot.equals("OnFailToOpen") ) return "OnFailToOpen";
    }
    if( resname.endsWith("ute") ) {
      // Encounter
      if( slot.equals("OnEnter") ) return "OnEntered";
      if( slot.equals("OnExhausted") ) return "OnExhausted";
      if( slot.equals("OnExit") ) return "OnExit";
      if( slot.equals("OnHeartbeat") ) return "OnHeartbeat";
      if( slot.equals("OnUserDefined") ) return "OnUserDefined";
    }
    if( resname.endsWith("uti") ) {
      // Item
    }
    if( resname.endsWith("utm") ) {
      // Store
      if( slot.equals("OnOpenStore") ) return "OnOpenStore";
    }
    if( resname.endsWith("utp") ) {
      // Placeable
      if( slot.equals("OnClose") ) return "OnClosed";
      if( slot.equals("OnDamaged") ) return "OnDamaged";
      if( slot.equals("OnDeath") ) return "OnDeath";
      if( slot.equals("OnHeartbeat") ) return "OnHeartbeat";
      if( slot.equals("OnDisturbed") ) return "OnInvDisturbed";
      if( slot.equals("OnLock") ) return "OnLock";
      if( slot.equals("OnPhysicalAttacked") ) return "OnMeleeAttacked";
      if( slot.equals("OnOpen") ) return "OnOpen";
      if( slot.equals("OnSpellCastAt") ) return "OnSpellCastAt";
      if( slot.equals("OnUnLock") ) return "OnUnlock";
      if( slot.equals("OnUsed") ) return "OnUsed";
      if( slot.equals("OnUserDefined") ) return "OnUserDefined";
      if( slot.equals("OnDisarm") ) return "OnDisarm";
      if( slot.equals("OnTrapTriggered") ) return "OnTrapTriggered";
    }
    if( resname.endsWith("uts") ) {
      // Sound
    }
    if( resname.endsWith("utt") ) {
      // Trigger
      if( slot.equals("OnClick") ) return "OnClick";
      if( slot.equals("OnEnter") ) return "ScriptOnEnter";
      if( slot.equals("OnExit") ) return "ScriptOnExit";
      if( slot.equals("OnHeartbeat") ) return "ScriptHeartbeat";
      if( slot.equals("OnUserDefined") ) return "ScriptUserDefine";
      if( slot.equals("OnDisarm") ) return "OnDisarm";
      if( slot.equals("OnTrapTriggered") ) return "OnTrapTriggered";
    }
    if( resname.endsWith("utw") ) {
      // Waypoint
    }

    throw new Exception("Unrecognized slot: ("+resname+")("+slot+")");
      
       
  }

}
