/* -*- c-basic-offset: 2 -*- */

package nwnintf;
import java.io.*;
import java.util.*;
import nwnintf.format.*;
import nwnintf.format.mod.*;
import nwnintf.format.itp.*;
import nwnintf.format.itp.element.*;


/**
 * Represent and maintain the list of blueprint slots and scripts that
 * have been inserted by scriptease into the module. Sit on top of
 * the .sez ItpFile, particulary the <root>.ResList element.
 */

public class SezFile {

  public static ResRef SEZRES = new ResRef("scripteaseDATA", ResRef.tiPTT );

  public static String RESLISTELT = "ResList";
  public static String INSTELT = "Instances";


  NwnModule nwnmodule_;
  ItpFile itp_;
  ItpListElt reslist_;
  ItpDatrefElt instdat_;

  public SezFile(NwnModule nwn) throws Exception {
    nwnmodule_ = nwn;

    byte[] sezdata = nwnmodule_.getModfile().resource( SEZRES );
    if( sezdata == null ) {
      /* First time.
       */
      itp_ = new ItpFile( "SEZ ", "V3.2" );
    } else {
      /* Already exists.
       */
      itp_ = new ItpFile( sezdata );
    }

    reslist_ = (ItpListElt)itp_.root().element(RESLISTELT);
    if( reslist_ == null ) {
      reslist_ = new ItpListElt( itp_ );
      itp_.addElement(reslist_);
      itp_.root().addElement(reslist_);
      reslist_.setName(RESLISTELT);
    }

    instdat_ = (ItpDatrefElt)itp_.root().element(INSTELT);
    if( instdat_ == null ) {
      instdat_ = new ItpDatrefElt( itp_ );
      itp_.addElement(instdat_);
      itp_.root().addElement(instdat_);
      instdat_.setName(INSTELT);
    }

  }

  public void removeFromReslist(ItpEntry entry) {

    itp_.removeEntry( entry );
    ((ItpListElt)itp_.root().element(RESLISTELT)).removeEntry( entry );
  }

  public ItpEntry newReslistEntry() {
    ItpEntry ent = new ItpEntry( itp_ );
    reslist_.addEntry(ent);
    itp_.addEntry(ent);
    return ent;
  }

  public ItpListElt reslist() {
    return reslist_;
  }

  public ItpFile itp() {
    return itp_;
  }

  public byte[] getInstanceData() {
    return (byte[])instdat_.getData().clone();
  }
  public void setInstanceData(byte[] data) {
    instdat_.setData(data);
  }

  public void save() throws Exception {
    byte[] resbytes = new ItpWriter( itp_ ).bytes();
    nwnmodule_.getModfile().addupdateresource( SEZRES, resbytes );
    
  }

}
