/* -*- c-basic-offset: 2 -*- */
package nwnintf.conversation;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.io.IOException;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import nwnintf.format.*;
import nwnintf.format.itp.*;
import nwnintf.format.itp.element.*;
import nwnintf.format.tlk.*;


public class ConversationPanel extends JPanel {
    
  ConversationTree ctree_;
  JTree jtree_;

  public ConversationPanel( ConversationTree ctree ) {
    ctree_ = ctree;

    jtree_ = new JTree( ctree_.root() );
    jtree_.getSelectionModel().setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
    jtree_.setExpandsSelectedPaths(true);
    //jtree_.setRootVisible(false);
    jtree_.setShowsRootHandles(true);

    DefaultTreeCellRenderer renderer = new ConversationCellRenderer();//DefaultTreeCellRenderer();
    renderer.setLeafIcon(null);
    renderer.setOpenIcon(null);
    renderer.setClosedIcon(null);
    jtree_.setCellRenderer(renderer);

    // Completely expand the tree.
    for( Enumeration it = ctree_.enumeration(); it.hasMoreElements(); ) {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)it.nextElement();
      jtree_.expandPath( new TreePath(node.getPath()) );
    }

    JScrollPane treeview = new JScrollPane( jtree_ );
    treeview.setMinimumSize( new Dimension(100,50) );
    treeview.setPreferredSize( new Dimension( 400, 200 ) );
    setLayout( new GridLayout(1,1) );
    add( treeview, BorderLayout.CENTER );
  }


  /**
   * Return the current selection.
   */
  public ConversationNode getSelection() {
    DefaultMutableTreeNode node =
      (DefaultMutableTreeNode)jtree_.getLastSelectedPathComponent();
    if( node != null ) {
      ConversationNode cn = (ConversationNode)node.getUserObject();
      return cn;

    }
    return null;
  }

  public void setSelection( ConversationNode cn ) {
    if( cn != null ) {
      setSelection(cn.entry());
    } else {
      setSelection((ItpEntry)null);
    }
  }

  /**
   * Set the current selection.
   * @param ent If null, select the root node.
   */
  public void setSelection( ItpEntry ent ) {
    for( Enumeration e = ctree_.enumeration(); e.hasMoreElements(); ) {
      DefaultMutableTreeNode tn = (DefaultMutableTreeNode)e.nextElement();
      if( tn != null ) {
	ConversationNode cn = (ConversationNode)tn.getUserObject();
	if( cn != null && cn.entry() == ent ) {
	  TreePath tp = new TreePath(tn.getPath());
	  jtree_.setSelectionPath(tp);
	  jtree_.scrollPathToVisible(tp);
	  return;
	}
      }
    }
    jtree_.setSelectionPath(null);
  }

  /**
   * The actionCommand of the ActionEvent sent to the listener will be
   * nothing. You have to check back with the tree to see what, if anything,
   * is selected.
   */

  public void addSelectionListener( ActionListener listener ) {
    jtree_.addTreeSelectionListener( 
        new ConversationSelectionThunk( this, listener ) );
  }

}

/**
 * Receive TreeSelectionEvent from the tree, transform into
 * ActionEvent, and forward to an ActionListener.
 */
class ConversationSelectionThunk implements TreeSelectionListener {
  ConversationPanel conv_;
  ActionListener listener_;
  ConversationSelectionThunk( ConversationPanel conv,
			      ActionListener listener ) {
    conv_ = conv;
    listener_ = listener;
  }

  public void valueChanged( TreeSelectionEvent evt ) {

    ActionEvent aevt =
      new ActionEvent(this, ActionEvent.ACTION_PERFORMED,null );

    listener_.actionPerformed(aevt);
	  
  }

}
