/* -*- c-basic-offset: 2 -*- */
package nwnintf.conversation;

import java.util.*;

import javax.swing.tree.*;
import nwnintf.format.*;
import nwnintf.format.itp.*;
import nwnintf.format.itp.element.*;
import nwnintf.format.tlk.*;

/**
 * Wrap an itp dialog file into a swing tree.
 */
public class ConversationTree  {

  public static int SPEAKER_PC = 0;
  public static int SPEAKER_NPC = 1;

  DefaultMutableTreeNode treeroot_;
  ItpFile itp_;
  TlkFile tlk_;

  public ConversationTree( ItpFile itp, TlkFile tlk )
			  throws java.io.IOException
  {
    itp_ = itp;
    tlk_ = tlk;
    treeroot_ = new DefaultMutableTreeNode();

    makenodes( treeroot_, (ItpListElt)(itp.root().element("StartingList")),
	       SPEAKER_NPC );
  }


  private int other( int speaker ) { 
    if( speaker == SPEAKER_PC ) {
      return SPEAKER_NPC;
    } else {
      // speaker == SPEAKER_NPC
      return SPEAKER_PC;
    }
  }

  /**
   * Return the list of all conversation nodes available to the speaker.
   */
  private ItpListElt allConvList( int speaker ) {
    if( speaker == SPEAKER_NPC ) {
      return (ItpListElt)(itp_.root().element("EntryList"));
    } else {
      // speaker == SPEAKER_PC
      return (ItpListElt)(itp_.root().element("ReplyList"));
    }
  }

  /**
   * Return the list of replies to the speaker's conversation node.
   */
  private ItpListElt responseList( ItpEntry ent, int speaker ) {
    if( speaker == SPEAKER_PC ) {
      return (ItpListElt)(ent.element("EntriesList"));
    } else {
      // speaker == SPEAKER_NPC
      return (ItpListElt)(ent.element("RepliesList"));
    }
  }


  /**
   * @param treenode Root of the children we are going to add here.
   * @param entrylist References to response conversation nodes.
   * @param speaker Speaker type of the responses to treenode.
   */
  private void makenodes( DefaultMutableTreeNode treenode,
			  ItpListElt entrylist, int speaker ) 
			  throws java.io.IOException
  {

    for(int i = 0; i < entrylist.length(); ++i ) {

      ItpEntry ent = (ItpEntry)entrylist.entry(i);

      int index = ((ItpUint32Elt)ent.element("Index")).value();
      boolean isChild;
      ItpUint8Elt elt8 = (ItpUint8Elt)ent.element("IsChild");
      if( elt8 != null ) { isChild = (elt8.value() != 0); }
      else { isChild = false; }

      ItpEntry responseEnt = allConvList( speaker ).entry(index);
      ItpListElt childList = responseList(responseEnt, speaker);

      ConversationNode cn =
	new ConversationNode( tlk_, responseEnt,
			      speaker, isChild,
			      (childList.length() == 0));
      DefaultMutableTreeNode newnode =
	new DefaultMutableTreeNode( cn );

      treenode.add( newnode );

      if( !isChild ) {
	makenodes( newnode, childList, other(speaker) );
      }
    }
  }

  TreeNode root() { return treeroot_; }

  public Enumeration enumeration() {
    return treeroot_.breadthFirstEnumeration();
  }

}
