/* -*- c-basic-offset: 2 -*- */

package nwnintf.example;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.util.*;

import nwnintf.*;
import nwnintf.format.*;
import nwnintf.format.mod.*;
import nwnintf.format.itp.*;
import nwnintf.format.itp.element.*;
import nwnintf.palette.*;
import nwnintf.conversation.*;

public class ExamplePanel extends JPanel {

  NwnModule module_;

  JLabel bluelabel_;
  MultiPalette palette_;

  ConvListPanel convlist_;
  JPanel convholder_;
  ConversationPanel conversation_;

  GridBagLayout gridbag_;

  public ExamplePanel(NwnModule module) {
    module_ = module;

    /* Swing layout is not fun. Gridbag overkill, you say? I would
     * agree, but the alternative is worse.
     */
    gridbag_ = new GridBagLayout();
    setLayout(gridbag_);

    makePalette();
    makePaletteLabel();
    makeConvListPanel();

    GridBagConstraints c = new GridBagConstraints();

    /* Blueprint palette in column 1 row 1.
     */
    c.gridwidth = 1;
    c.gridx = 0;
    c.gridy = 0;
    c.fill = GridBagConstraints.NONE;
    c.weightx = 0;
    gbadd( palette_, c );

    /* Label describing current selection in palette, column 1, row 2.
     */
    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    c.gridy = 1;
    gbadd(bluelabel_, c);

    /* List of available conversation files column2, row 1+2.
     */
    c.fill = GridBagConstraints.VERTICAL;
    c.weightx = 0;
    c.weighty = 1;
    c.gridy = 0;
    c.gridx = 1;
    c.gridheight = 2;
    gbadd( convlist_, c);

    /* Selected conversation, column 3, row 1+2
     */
    convholder_ = new JPanel();
    convholder_.setPreferredSize( new Dimension(400,200) );
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1;
    c.weighty = 1;
    c.gridy = 0;
    c.gridx = 2;
    c.gridheight = 2;
    gbadd( convholder_, c );
  }


  private void makePalette() {
    Collection enabledtabs = new Vector();
    enabledtabs.add("utc"); enabledtabs.add("utd"); enabledtabs.add("utp");
    enabledtabs.add("ute"); enabledtabs.add("uts"); enabledtabs.add("utm");
    enabledtabs.add("uti"); enabledtabs.add("utt"); enabledtabs.add("utw");
    palette_ = new MultiPalette( module_.getMultiPaletteTree(), enabledtabs );
  }

  private void makePaletteLabel() {
    bluelabel_ = new JLabel("Selected blueprint: <none>",
			    SwingConstants.LEFT );
    bluelabel_.setLabelFor(palette_);
    //bluelabel_.setBorder( BorderFactory.createEtchedBorder(javax.swing.border.EtchedBorder.RAISED));
    palette_.addSelectionListener(new ActionListener() {
	public void actionPerformed(ActionEvent evt) {
	  ResRef r = palette_.getSelection();
	  if( r == null ) bluelabel_.setText("Selected blueprint: <none>");
	  else bluelabel_.setText( "Selected blueprint: \"" + r.toString() +"\"" );
	}
      });
  }

  private void makeConvListPanel() {
    convlist_ = new ConvListPanel(module_);
    convlist_.addConvSelectionListener( new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  ResRef r = convlist_.getSelectedResRef();

	  if( conversation_ != null) {
	    convholder_.remove( conversation_ );
	    conversation_ = null;
	  }
	  if( r != null ) {
	    try {
	      ConversationTree tree =
		new ConversationTree(new ItpFile( module_.getModfile().
						  resource( r ) ),
				     module_.getTlkfile());
	      conversation_ = new ConversationPanel( tree );

	      convholder_.add(conversation_);
	      convholder_.validate();
	    } catch( Exception exc ) {
	      exc.printStackTrace();
	    }
	  }
	}
      });
  }

  private void gbadd( Component comp, GridBagConstraints c ) {
    gridbag_.setConstraints( comp, c );
    add( comp );
  }

}
