/* -*- c-basic-offset: 2 -*- */

package nwnintf.format;

import java.io.RandomAccessFile;
import java.io.IOException;

/**
 * Provide static methods for decoding types from bytes.
 */
public class Prim {
    
  public static int int32lsb( byte b1, byte b2, byte b3, byte b4 ) {
    return
      ((b1 < 0 ? b1 + 256 : b1)) | 
      ((b2 < 0 ? b2 + 256 : b2) << 8) |
      ((b3 < 0 ? b3 + 256 : b3) << 16) |
      ((b4 < 0 ? b4 + 256 : b4) << 24);
  }

  public static int int32lsb( byte[] bytes, int offset ) {

    return int32lsb( bytes[offset+0], bytes[offset+1],
		     bytes[offset+2], bytes[offset+3] );

  }

  public static long int64lsb( byte[] bytes, int offset ) {
    long result = 0;
    int shiftby = 56;

    for( int i = 7; i >= 0; --i, shiftby -= 8 ) {
      long x = bytes[offset+i];
      if( x < 0 ) x = 256 + x;
      result |= x << shiftby;
    }
    return result;
  }

  /**
   * This returns int so that it can encompass the whole range of uint8.
   * Could return short too, but whatever.
   */
  public static int uint8( byte[] bytes, int offset ) {
    int result = bytes[offset];
    if( result < 0 ) result = 256 + result;
    return result;
  }

  /**
   * This returns int so that it can encompass the whole range of uint16.
   */
  public static int uint16lsb( byte[] bytes, int offset ) {
    int result = 0;
    int b;
    b = bytes[offset];
    if( b < 0 ) b = 256 + b;
    result |= b << 8;
    b = bytes[offset+1];
    if( b < 0 ) b = 256 + b;
    result |= b;
    
    return result;
  }



  /**
   * Return a string of a given length, but stopping at null character.
   */
  public static String string0( byte[] bytes, int offset, int length ) {
    StringBuffer sb = new StringBuffer(length);

    for(int i = 0; i < length; ++i ) {

      byte b = bytes[offset+i];

      if( b == 0 ) break;
      sb.append((char)b);
    }
    return sb.toString();
  }

  public static String string( byte[] bytes, int offset, int length ) {
    StringBuffer sb = new StringBuffer(length);

    for(int i = 0; i < length; ++i ) {
      byte b = bytes[offset+i];

      sb.append((char)b);
    }
    return sb.toString();
  }

  public static void main(String[] args) {
    byte[] b = { 0, 0, 0, -128 };
    System.out.println( int32lsb( b, 0 ) -1);
    
    byte[] b2 = { 0, 0, 0, 0, 0, 0, 0, -128 };
    System.out.println( int64lsb( b2, 0 ) );
    System.out.println( int64lsb( b2, 0 ) - 1);
    System.out.println( Long.MAX_VALUE );
    System.out.println( Long.MIN_VALUE );
  }

  public static String string(RandomAccessFile f, int length) throws java.io.IOException {
    byte[] bytes = new byte[length];
    f.readFully(bytes);
    return Prim.string(bytes,0,length);
  }
  /**
   * Read length bytes but return value stops at null byte.
   */
  public static String string0(RandomAccessFile f, int length) throws java.io.IOException {

    byte[] bytes = new byte[length];
    f.readFully(bytes);
    StringBuffer sb = new StringBuffer(length);

    for(int i = 0; i < length; ++i ) {

      if( bytes[i] == 0 ) break;
      sb.append((char)bytes[i]);
    }
    return sb.toString();
  }
    

  public static int int32lsb(RandomAccessFile f) throws java.io.IOException {
    byte b1, b2, b3, b4;
    b1 = f.readByte();
    b2 = f.readByte();
    b3 = f.readByte();
    b4 = f.readByte();
    return int32lsb( b1,b2,b3,b4 );
  }

  public static void writeIntLSB(RandomAccessFile f, int x) throws IOException {
    f.write( 0xff & x );
    f.write( 0xff & (x >> 8) );
    f.write( 0xff & (x >> 16) );
    f.write( 0xff & (x >>> 24) );
  }

  public static void writeBytes0(RandomAccessFile f, String x, int len ) throws IOException {
    if( x.length() > len ) {
      x = x.substring(0,len);
    }
    f.writeBytes( x );
    for( int i = 0; i < len - x.length(); ++i ) {
      f.write(0);
    }
  }
}
