/* -*- c-basic-offset: 2 -*- */

package nwnintf.format.itp;

import java.util.Vector;
import java.io.IOException;

import nwnintf.format.Prim;
import nwnintf.format.itp.element.*;
/**
 * Represent an "element" in the itp file.
 */
abstract public class ItpElement {

  public static int TYPE_UINT8 = 0;
  public static int TYPE_INT8  = 1;
  public static int TYPE_UINT16  = 2;
  public static int TYPE_INT16 = 3;
  public static int TYPE_UINT32 = 4;
  public static int TYPE_INT32 = 5;
  public static int TYPE_UINT64 = 6;
  public static int TYPE_INT64 = 7;
  public static int TYPE_FLOAT32 = 8;
  public static int TYPE_FLOAT64 = 9;
  public static int TYPE_STRING = 10;
  public static int TYPE_RESREF = 11;
  public static int TYPE_STRREF = 12;
  public static int TYPE_DATREF = 13;
  public static int TYPE_CAPREF = 14;
  public static int TYPE_LIST = 15;

  public static int SIZE = 12;
  public static int TYPE = 0;
  public static int NAMEIDX = 4;
  public static int DATA = 8;


  ItpFile f_;
  String name_;

  public ItpElement( ItpFile f ) {
    f_ = f;
  }

  public void read( byte[] bytes, int eltoffset, int varnameoffset,
		    int vardataoffset, int listoffset ) {

    int nameidx = Prim.int32lsb( bytes, eltoffset + NAMEIDX );
    int nameoffset = varnameoffset + nameidx * 16;

    name_ = Prim.string0( bytes, nameoffset, 16 );
    
  }


  public static ItpElement newElement( ItpFile f, byte[] bytes, int offset ) 
    throws Exception {
    int type = Prim.int32lsb( bytes, offset + TYPE );


    if     ( type == TYPE_UINT8 ) return new ItpUint8Elt(f);
    else if( type == TYPE_INT8 ) return new ItpInt8Elt(f);
    else if( type == TYPE_UINT16 ) return new ItpUint16Elt(f);
    else if( type == TYPE_INT16 ) return new ItpInt16Elt(f);
    else if( type == TYPE_UINT32 ) return new ItpUint32Elt(f);
    else if( type == TYPE_INT32 ) return new ItpInt32Elt(f);
    else if( type == TYPE_UINT64 ) return new ItpUint64Elt(f);
    else if( type == TYPE_INT64 ) return new ItpInt64Elt(f);
    else if( type == TYPE_FLOAT32 ) return new ItpFloat32Elt(f);
    else if( type == TYPE_FLOAT64 ) return new ItpFloat64Elt(f);
    else if( type == TYPE_STRING ) return new ItpStringElt(f);
    else if( type == TYPE_RESREF ) return new ItpResrefElt(f);
    else if( type == TYPE_STRREF ) return new ItpStrrefElt(f);
    else if( type == TYPE_DATREF ) return new ItpDatrefElt(f);
    else if( type == TYPE_CAPREF ) return new ItpCaprefElt(f);
    else if( type == TYPE_LIST ) return new ItpListElt(f);

    throw new Exception("Unknown type");
    
  }

  protected ItpFile file() {
    return f_;
  }

  public String name() {
    return name_;
  }

  public void setName( String name ) {
    name_ = name;
  }

  abstract public int type();
  /**
   * Write the element type-specific data to ItpWriter and return the
   * contents that should go in the data field in the element
   * structure for output.
   */
  abstract protected int childData(ItpWriter w) throws IOException;

  /**
   * Subclasses that contain other entries or elements should
   * override this.
   */
  public void removeChildren() {
  }

  public void write( ItpWriter w ) throws IOException {
    w.writeElement( type(), name(), childData(w) );
  }

}
