/* -*- c-basic-offset: 2 -*- */

package nwnintf.format.itp;

import nwnintf.format.Prim;
import java.util.*;

/**
 * Represent an "entry" in the itp file.
 */
public class ItpEntry {

  public static int SIZE = 12;
  public static int CODE = 0;
  public static int INDEXOROFFSET = 4;
  public static int NELTS = 8;

  ItpFile f_;
  
  int code_;
  Vector elements_;

  public ItpEntry( ItpFile f ) {
    f_ = f;
    code_ = 0;
    elements_ = new Vector();
  }

  public void read( byte[] bytes, int offset, int multimapoffset ) {
    code_ = Prim.int32lsb( bytes, offset + CODE );
    int indexoroffset = Prim.int32lsb( bytes, offset + INDEXOROFFSET );
    int nelements = Prim.int32lsb( bytes, offset + NELTS );

    elements_ = new Vector(nelements);

    if( nelements == 1 ) {

      elements_.add( f_.element( indexoroffset ) );

    } else if( nelements > 1 ) {

      for( int i = 0; i < nelements; ++i ) {
	int eltidx = Prim.int32lsb(bytes, 
				   multimapoffset + indexoroffset + 4 * i);
	elements_.add( f_.element( eltidx ) );
      }
      
    }
  }

  public ItpElement element(int eltidx) {
    return (ItpElement)elements_.elementAt(eltidx);
  }

  public ItpElement element(String name) {
    for( java.util.Enumeration e = elements_.elements();
	 e.hasMoreElements(); ) {
      ItpElement elt = (ItpElement)e.nextElement();
      if( elt.name().equals(name) ) return elt;
    }
    return null;
  }

  public int length() {
    return elements_.size();
  }

  /**
   * Add a new element.
   * Remember to add it to the ItpFile as well.
   */
  public void addElement(ItpElement elt) {
    elements_.add(elt);
  }

  public void removeChildren() {
    for( Iterator it = elements_.iterator(); it.hasNext(); ) {
      f_.removeElement( (ItpElement)it.next() );
    }
    elements_ = new Vector();
  }


  public void write( ItpWriter w ) throws java.io.IOException {

    w.entries().writeInt( code_ );

    if( elements_.size() == 1 ) {

      ItpElement elt = (ItpElement)elements_.elementAt( 0 );
      w.entries().writeInt( f_.indexOfElement( elt ) );

    } else {

      int multimapoffset = w.multimap().size();
      for( java.util.Enumeration e = elements_.elements();
	   e.hasMoreElements(); ) {
	ItpElement elt = (ItpElement)e.nextElement();

	w.multimap().writeInt( f_.indexOfElement(elt) );
      }

      w.entries().writeInt( multimapoffset );
    }

    w.entries().writeInt(elements_.size());
  }

}
