/* -*- c-basic-offset: 2 -*- */

package nwnintf.format.itp;

import java.io.*;

/**
 * Can't implement DataOutput because that interface specifies MSB
 * byte order. I only defined ItpStream because I need to write in LSB.
 */
public class ItpStream {

  FilterOutputStream fos_;
  int size_;

  public ItpStream(OutputStream os) {
    fos_ = new FilterOutputStream(os);
    size_ = 0;
  }

  /**
   * Write an int in LSB order to the underlying stream.
   */
  public void writeInt(int x) throws IOException {
    fos_.write( 0xff & x ); size_++;
    fos_.write( 0xff & (x >> 8) ); size_++;
    fos_.write( 0xff & (x >> 16) ); size_++;
    fos_.write( 0xff & (x >>> 24) ); size_++;
  }
  
  public void writeByte(int x) throws IOException {
    fos_.write( x );
    size_++;
  }

  /**
   * Write the low byte of each character of s to the underlying stream.
   */
  public void writeBytes(String s) throws IOException {
    int len = s.length();
    for( int i = 0; i < len; ++i ) {
      writeByte( (byte)(s.charAt(i) & 0xff) );
    }
  }

  /**
   * Write a double in LSB order.
   */
  public void writeDouble( double x ) throws IOException {
    writeLong( Double.doubleToLongBits(x) );
  }

  /**
   * Write a long in LSB order.
   */
  public void writeLong( long x ) throws IOException {
    fos_.write( (int)(0xff & x )); size_++;
    fos_.write( (int)(0xff & (x >>> 8) )); size_++;
    fos_.write( (int)(0xff & (x >>> 16) )); size_++;
    fos_.write( (int)(0xff & (x >>> 24) )); size_++;
    fos_.write( (int)(0xff & (x >>> 32) )); size_++;
    fos_.write( (int)(0xff & (x >>> 40) )); size_++;
    fos_.write( (int)(0xff & (x >>> 48) )); size_++;
    fos_.write( (int)(0xff & (x >>> 56) )); size_++;
  }

  public void flush() throws IOException {
    fos_.flush();
  }


  public void writeBytes(byte[] b) throws IOException {
    size_+=b.length;
    fos_.write(b);
  }
  

  /**
   * Return the number of bytes written to the underlying stream by this
   * ItpStream instance.
   */
  public int size() {
    return size_;
  }

}
