/* -*- c-basic-offset: 2 -*- */

package nwnintf.format.itp;

import java.util.Vector;
import java.util.HashMap;
import java.util.Map;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.DataOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutput;
import java.io.OutputStream;

public class ItpWriter {

  ItpFile f_;
  Buf header_;
  Buf entries_;
  Buf elements_;
  Buf varnames_;
  Buf vardata_;
  Buf multimap_;
  Buf list_;

  Map namemap_;

  byte[] result_;

  public ItpWriter(ItpFile f) {
    f_ = f;
    header_ = new Buf();
    entries_ = new Buf();
    elements_ = new Buf();
    varnames_ = new Buf();
    vardata_ = new Buf();
    multimap_ = new Buf();
    list_ = new Buf();
    namemap_ = new HashMap();
  }

  public ItpStream header() { return header_.itpStream(); }
  public ItpStream entries() { return entries_.itpStream(); }
  public ItpStream elements() { return elements_.itpStream(); }

  /**
   * Don't call this to add a new variable name. Use varname(String).
   * so the names can be interned.
   * @see varname(String)
   */
  public ItpStream varnames() { return varnames_.itpStream(); }

  public ItpStream vardata() { return vardata_.itpStream(); }
  public ItpStream multimap() { return multimap_.itpStream(); }
  public ItpStream list() { return list_.itpStream(); }

  /**
   * Add a new variable name to the file.
   * @param s Variable name
   * @return Index of variable name sectin.
   */
  public int varname(String s) throws IOException {
    Integer idx = (Integer)namemap_.get(s);
    if( idx == null ) {
      idx = new Integer( namemap_.size() );
      varnames().writeBytes(s);
      for(int i = 16 - s.length(); i > 0; --i ) {
	varnames().writeByte( 0 );
      }
      namemap_.put( s, idx );
    }
    return idx.intValue();
  }

  public int nvarnames() {
    return namemap_.size();
  }

  public void writeElement(int type, String name, int data) 
    throws IOException {
    elements().writeInt( type );
    elements().writeInt( varname(name) );
    elements().writeInt( data );
  }

  /**
   * Serialize this ItpFile to a sequence of bytes.
   * @return The sequence of bytes representing this ItpFile in bioware's
   * ITP format.
   */
  public byte[] bytes() throws Exception {
    if( result_ != null ) return result_;


    for( int i = 0; i < f_.nentries(); ++i ) {
      f_.entry(i).write(this);
    }
    for( int i = 0; i < f_.nelements(); ++i ) {
      f_.element(i).write(this);
    }

    header().writeBytes(f_.type());
    header().writeBytes(f_.version());

    int offset = ItpFile.SIZE;

    // entryoffset
    header().writeInt( offset );
    // nentries
    header().writeInt( f_.nentries() );
    offset += entries().size();

    // elementoffset
    header().writeInt( offset );
    // nelements
    header().writeInt( f_.nelements() );
    offset += elements().size();

    // varnameoffset
    header().writeInt( offset );
    // nvarnames
    header().writeInt( nvarnames() );
    offset += varnames().size();

    // vardataoffset
    header().writeInt( offset );
    // nvardatabytes
    header().writeInt( vardata().size() );
    offset += vardata().size();

    // multimapoffset
    header().writeInt( offset );
    // nmultimapbytes
    header().writeInt( multimap().size() );
    offset += multimap().size();

    // listoffset
    header().writeInt( offset );
    // nlistbytes
    header().writeInt( list().size() );
    offset += list().size();

    //System.out.println("supposedly wrote " + Integer.toString(offset));

    java.io.ByteArrayOutputStream bos = new java.io.ByteArrayOutputStream();

    bos.write( header_.bytes() );
    bos.write( entries_.bytes() );
    bos.write( elements_.bytes() );
    bos.write( varnames_.bytes() );
    bos.write( vardata_.bytes() );
    bos.write( multimap_.bytes() );
    bos.write( list_.bytes() );

    result_ = bos.toByteArray();
    return result_;
  }

}



class Buf /*implements DataOutput*/ {
  ByteArrayOutputStream baos_;
  ItpStream itps_;

  Buf() {
    baos_ = new ByteArrayOutputStream();
    itps_ = new ItpStream(baos_);
  }

  ItpStream itpStream() { return itps_; }
  ByteArrayOutputStream byteArrayOutputStream() { return baos_; }
  byte[] bytes() throws IOException { 
    itps_.flush();
    byte[] result =  baos_.toByteArray(); 
    if( itps_.size() != result.length ) {
      throw new IOException("JACKASS " + Integer.toString(itps_.size()) + " " +
			    Integer.toString(result.length));
    }
    return result;
  }
}
