/* -*- c-basic-offset: 2 -*- */

package nwnintf.format.itp.element;

import nwnintf.format.*;
import nwnintf.format.itp.*;
import java.util.*;

public class ItpListElt extends ItpElement {

  Vector children_;

  public ItpListElt( ItpFile f ) {
    super(f);
    children_ = new Vector();
  }

  public void read( byte[] bytes, int eltoffset, int varnameoffset,
                    int vardataoffset, int listoffset ) {
    super.read( bytes, eltoffset, varnameoffset, vardataoffset, listoffset );

    children_ = new Vector();

    int childlistoffsetinlist = Prim.int32lsb( bytes, eltoffset + DATA );

    /* Child list is a 4-byte count of children followed by that many
     * entry index numbers.
     */
    int childlistoffset = listoffset + childlistoffsetinlist;
    int nchildren = Prim.int32lsb( bytes, childlistoffset );

    for( int i = 0; i < nchildren; ++i ) {
      int childentidx = Prim.int32lsb( bytes, childlistoffset + i * 4 + 4 );
      ItpEntry child = file().entry(childentidx);
      children_.add(child);
    }
    
  }
  
  public ItpEntry entry(int entidx) {
    return (ItpEntry)children_.elementAt(entidx);
  }
  
  public int length() {
    return children_.size();
  }

  public int type() {
    return TYPE_LIST;
  }

  protected int childData(ItpWriter w) throws java.io.IOException {
    int data = w.list().size();
    w.list().writeInt( children_.size() );
    for( Enumeration e = children_.elements(); e.hasMoreElements(); ) {
      w.list().writeInt( file().indexOfEntry( (ItpEntry)e.nextElement() ) );
    }
    return data;
  }

  public void removeChildren() {
    super.removeChildren();
    for( Iterator it = children_.iterator(); it.hasNext(); ) {
      file().removeEntry( (ItpEntry)it.next() );
    }
    children_ = new Vector();
  }

  /**
   * Add a new element.
   * Remember to add it to the ItpFile as well.
   */
  public void addEntry(ItpEntry ent) {
    children_.add(ent);
  }

  public void removeEntry(ItpEntry ent) {
    children_.remove(ent);
  }
}
