/* -*- c-basic-offset: 2 -*- */

package nwnintf.format.itp.element;

import java.io.IOException;
import nwnintf.format.*;
import nwnintf.format.itp.*;
import nwnintf.format.tlk.TlkFile;

public class ItpStrrefElt extends ItpElement {
  public static int NBYTES = 0;
  public static int DIALOGTLKID = 4;
  public static int NSTRINGS = 8;

  int dialogtlk_;
  private Langstring[] strings_;

  public ItpStrrefElt( ItpFile f ) {
    super(f);
    dialogtlk_ = -1;
    strings_ = new Langstring[0];
  }
  public void read( byte[] bytes, int eltoffset, int varnameoffset,
                    int vardataoffset, int listoffset ) {
    super.read( bytes, eltoffset, varnameoffset, vardataoffset, listoffset );

    int offset = vardataoffset + Prim.int32lsb( bytes, eltoffset + DATA );
    int nbytes = Prim.int32lsb( bytes, offset + NBYTES );
    dialogtlk_ = Prim.int32lsb( bytes, offset + DIALOGTLKID );
    int nstrings = Prim.int32lsb( bytes, offset + NSTRINGS );

    strings_ = new Langstring[nstrings];
    int lsoffset = offset + 12;

    for(int i = 0; i < nstrings; ++i ) {

      strings_[i] = new Langstring( bytes, lsoffset );
      lsoffset += Langstring.lengthAt( bytes, lsoffset );
    }
  }
  public int type() {
    return TYPE_STRREF;
  }
  protected int childData(ItpWriter w) throws IOException {
    // First compute total size of structure, less 4.
    int nbytes = 8; // dialogtlk field and nstrings field.
    for(int i = 0; i < strings_.length; ++i ) {
      nbytes += strings_[i].size();
    }

    int data = w.vardata().size();
    w.vardata().writeInt(nbytes);
    w.vardata().writeInt(dialogtlk_);
    w.vardata().writeInt(strings_.length);
    for( int i = 0; i < strings_.length; ++i ) {
      strings_[i].write(w);
    }
    return data;
  }


  /**
   * Return the string represented by this strref in some indeterminate
   * language.
   */
  public String defaultValue(TlkFile tlk) throws java.io.IOException {
    if( dialogtlk_ >= 0 ) {
      return tlk.get(dialogtlk_);
    } else {
      if( strings_.length > 0 ) {
	return strings_[0].value();
      } else {
	return null;
      }
    }
  }

}

class Langstring {
  String value_;
  int language_;

  public Langstring( byte[] bytes, int lsoffset ) {
    // First the language type code.
    language_ = Prim.int32lsb( bytes, lsoffset );

    // Then the length
    int length = Prim.int32lsb( bytes, lsoffset + 4 );

    //System.out.println("Warning - ItpStrrefElt language-specific string untested");
    //System.out.println("length = " + Integer.toString(length));


    // Then the following string.
    value_ = Prim.string( bytes, lsoffset + 8, length );
    //System.out.println("value:|"+value_+"|");
  }

  public static int lengthAt( byte[] bytes, int lsoffset ) {
    int length = Prim.int32lsb( bytes, lsoffset + 4 );
    return 8 + length;
  }

  /**
   * The number of bytes this would take up if written.
   * @return size in bytes of this once written out to ITP file.
   */
  public int size() {
    return 8 + value_.length();
  }

  public void write(ItpWriter w) throws IOException {
    w.vardata().writeInt(language_);
    w.vardata().writeInt(value_.length());
    w.vardata().writeBytes(value_);
  }

  public String value() {
    return value_;
  }
}
