/* -*- c-basic-offset: 2 -*- */
package nwnintf.format.mod;

import java.util.*;

/**
 * Encapsulate a ResRef in a module.
 */
public class ResRef {
  // Type codes for common resource types.

  // Script source code
  public static String tsNSS = "nss";
  public static int    tiNSS = 0x07d9;
  // Script byte code
  public static String tsNCS = "ncs";
  public static int    tiNCS = 0x07da;
  // ScriptEase data
  public static String tsSEZ = "sez";
  public static int    tiSEZ = 0x5CEA2E;

  // Creature blueprint
  public static String tsUTC = "utc";
  public static int    tiUTC = 0x07eb;
  // Door blueprint
  public static String tsUTD = "utd";
  public static int    tiUTD = 0x07fa;
  // Encounter blueprint
  public static String tsUTE = "ute";
  public static int    tiUTE = 0x07f8;
  // Item blueprint
  public static String tsUTI = "uti";
  public static int    tiUTI = 0x07e9;
  // Merchant blueprint
  public static String tsUTM = "utm";
  public static int    tiUTM = 0x0803;
  // Placeable blueprint
  public static String tsUTP = "utp";
  public static int    tiUTP = 0x07fc;
  // Sound blueprint
  public static String tsUTS = "uts";
  public static int    tiUTS = 0x07f3;
  // Trigger blueprint
  public static String tsUTT = "utt";
  public static int    tiUTT = 0x07f0;
  // Waypoint blueprint
  public static String tsUTW = "utw";
  public static int    tiUTW = 0x080a;

  // Game Instance (area info)
  public static String tsGIT = "git";
  public static int    tiGIT = 2023;

  // Generic File Format (what I have been calling "ITP"
  public static String tsGFF = "gff";
  public static int    tiGFF = 2037;

  // Plot Manager Template (a gff)
  public static String tsPTT = "ptt";
  public static int    tiPTT = 2066;

  // Conversation aka dialog file
  public static String tsDLG = "dlg";
  public static int    tiDLG = 0x07ed;


  /**
   * Map string resource file type names to integer codes.
   */
  static private Map typemap_s2i_;
  /**
   * Map integer resource file type codes to string names.
   */
  static private Map typemap_i2s_;

  static {
    typemap_s2i_ = new HashMap();
    typemap_s2i_.put("res", new Integer(0x0000));
    typemap_s2i_.put("bmp", new Integer(0x0001));
    typemap_s2i_.put("mve", new Integer(0x0002));
    typemap_s2i_.put("tga", new Integer(0x0003));
    typemap_s2i_.put("wav", new Integer(0x0004));
    typemap_s2i_.put("plt", new Integer(0x0006));
    typemap_s2i_.put("ini", new Integer(0x0007));
    typemap_s2i_.put("bmu", new Integer(0x0008));
    typemap_s2i_.put("mpg", new Integer(0x0009));
    typemap_s2i_.put("txt", new Integer(0x000a));
    typemap_s2i_.put("plh", new Integer(0x07d0));
    typemap_s2i_.put("tex", new Integer(0x07d1));
    typemap_s2i_.put("mdl", new Integer(0x07d2));
    typemap_s2i_.put("thg", new Integer(0x07d3));
    typemap_s2i_.put("fnt", new Integer(0x07d5));
    typemap_s2i_.put("lua", new Integer(0x07d7));
    typemap_s2i_.put("slt", new Integer(0x07d8));
    typemap_s2i_.put(tsNSS, new Integer(tiNSS));
    typemap_s2i_.put(tsNCS, new Integer(tiNCS));
    typemap_s2i_.put("mod", new Integer(0x07db));
    typemap_s2i_.put("are", new Integer(0x07dc));
    typemap_s2i_.put("set", new Integer(0x07dd));
    typemap_s2i_.put("ifo", new Integer(0x07de));
    typemap_s2i_.put("bic", new Integer(0x07df));
    typemap_s2i_.put("wok", new Integer(0x07e0));
    typemap_s2i_.put("2da", new Integer(0x07e1));
    typemap_s2i_.put("tlk", new Integer(0x07e2));
    typemap_s2i_.put("txi", new Integer(0x07e6));
    typemap_s2i_.put("git", new Integer(0x07e7));
    typemap_s2i_.put("bti", new Integer(0x07e8));
    typemap_s2i_.put(tsUTI, new Integer(tiUTI));
    typemap_s2i_.put("btc", new Integer(0x07ea));
    typemap_s2i_.put(tsUTC, new Integer(tiUTC));
    typemap_s2i_.put(tsDLG, new Integer(tiDLG));
    typemap_s2i_.put("itp", new Integer(0x07ee));
    typemap_s2i_.put("btt", new Integer(0x07ef));
    typemap_s2i_.put(tsUTT, new Integer(tiUTT));
    typemap_s2i_.put("dds", new Integer(0x07f1));
    typemap_s2i_.put(tsUTS, new Integer(tiUTS));
    typemap_s2i_.put("ltr", new Integer(0x07f4));
    typemap_s2i_.put("gff", new Integer(0x07f5));
    typemap_s2i_.put("fac", new Integer(0x07f6));
    typemap_s2i_.put("bte", new Integer(0x07f7));
    typemap_s2i_.put(tsUTE, new Integer(tiUTE));
    typemap_s2i_.put("btd", new Integer(0x07f9));
    typemap_s2i_.put(tsUTD, new Integer(tiUTD));
    typemap_s2i_.put("btp", new Integer(0x07fb));
    typemap_s2i_.put(tsUTP, new Integer(tiUTP));
    typemap_s2i_.put("dtf", new Integer(0x07fd));
    typemap_s2i_.put("gic", new Integer(0x07fe));
    typemap_s2i_.put("gui", new Integer(0x07ff));
    typemap_s2i_.put("css", new Integer(0x0800));
    typemap_s2i_.put("ccs", new Integer(0x0801));
    typemap_s2i_.put("btm", new Integer(0x0802));
    typemap_s2i_.put(tsUTM, new Integer(tiUTM));
    typemap_s2i_.put("dwk", new Integer(0x0804));
    typemap_s2i_.put("pwk", new Integer(0x0805));
    typemap_s2i_.put("btg", new Integer(0x0806));
    typemap_s2i_.put("utg", new Integer(0x0807));
    typemap_s2i_.put("jrl", new Integer(0x0808));
    typemap_s2i_.put("sav", new Integer(0x0809));
    typemap_s2i_.put(tsUTW, new Integer(tiUTW));
    typemap_s2i_.put("4pc", new Integer(0x080b));
    typemap_s2i_.put("ssf", new Integer(0x080c));
    typemap_s2i_.put("hak", new Integer(0x080d));
    typemap_s2i_.put("nwm", new Integer(0x080e));
    typemap_s2i_.put("bik", new Integer(0x080f));
    typemap_s2i_.put("ptm", new Integer(0x0811));
    typemap_s2i_.put("erf", new Integer(0x270d));
    typemap_s2i_.put("bif", new Integer(0x270e));
    typemap_s2i_.put("key", new Integer(0x270f));
    typemap_s2i_.put(tsSEZ, new Integer(tiSEZ));

    typemap_i2s_ = new HashMap();
    for( Iterator it = typemap_s2i_.entrySet().iterator();
	 it.hasNext(); ) {
      Map.Entry ent = (Map.Entry)it.next();
      typemap_i2s_.put( ent.getValue(), ent.getKey() );
    }
  }



  String basename_;
  int type_;

  /**
   * No-argument constructor, for serialization.
   */
  public ResRef() {
    type_ = -1;
  }

  /**
   * @param basename The base name of the resref, e.g. "amauna"
   * @param type The type of the resref as an int. Use -1 for undefined type.
   */
  public ResRef( String basename, int type ) {
    basename_ = basename;
    type_ = type;
  }

  /**
   * @param basename The base name of the resref, e.g. "amauna".
   * @param type The type as a string, e.g. "utc", or as a number.
   */
  public ResRef( String basename, String type ) {
    basename_ = basename;
    type_ = typeInt(type);
  }

  /**
   * Copy constructor.
   */
  public ResRef( ResRef resref ) {
    basename_ = resref.basename_;
    type_ = resref.type_;
  }
  
  /**
   * Copy constructor, changing type.
   */
  public ResRef( ResRef resref, int type ) {
    basename_ = resref.basename_;
    type_ = type;
  }


  /**
   * @param nametype A dotted-pair resref e.g. "amauna.utc".
   */
  public static ResRef parseFilename( String nametype ) {
    String name;
    int type;

    int idx = nametype.lastIndexOf( '.' );

    if( idx == -1 ) {
      name = nametype;
      type = -1;
    } else {
      name = nametype.substring(0,idx);
      type = typeInt( nametype.substring(idx+1) );
    }
    return new ResRef( name, type );
  }

  /**
   * The integer type code for this string type.
   */
  public static int typeInt(String type) {
    if( type == null ) return -1;
    Integer i = (Integer)typemap_s2i_.get(type.toLowerCase());
    if( i == null ) {
      // Not found, try converting to integer.
      try {
	i = Integer.valueOf( type );
      } catch( NumberFormatException e ) {
	// Nope. Undefined, then.
	i = new Integer(-1);
      }
    }
    return i.intValue();
  }

  /**
   * Type of this resref as a string.
   * Null if undefined.
   */
  public String typeStr() {
    if( type_ == -1 ) return null;
    String result = (String)typemap_i2s_.get( new Integer(type_) );
    if( result == null ) {
      return Integer.toString(type_);
    } else {
      return result;
    }
  }

  /**
   * Type of this resref as an int.
   * -1 if undefined.
   */
  public int typeInt() {
    return type_;
  }

  /**
   * This resref as a filename in dotted-pair format, e.g. "monkey.utc"
   * Will throw if type is undefined.
   */
  public String filename() {
    return basename_ + "." + typeStr();
  }

  public void setBasename(String basename) {
    basename_ = basename;
  }
  public String getBasename() {
    return basename_;
  }

  /**
   * This method exists for serialization.
   * @see java.beans.XMLEncoder
   */
  public int getType() {
    return type_;
  }
  public void setType(int type) {
    type_ = type;
  }

  public void setType(String type) {
    type_ = typeInt(type);
  }

  
  public String name() {
    return basename_;
  }

  public boolean equals(Object o) {
    if( o == null ) return false;
    if( o instanceof ResRef ) {
      ResRef r = (ResRef)o;
      return
	r.name().equals( this.name() ) &&
	r.typeInt() == this.typeInt();
    }
    return false;
  }

  public int hashCode() {
    /* invert type so that higher-order bits of the hash code tend to be
     * very different when names are the same. I'm guessing that will
     * improve performance on hashtables since most type codes are different
     * only in the bottom few bits.
     */
    return name().hashCode() ^ ~type_;
  }

  public String toString() {
    return filename();
  }

}

