/* -*- c-basic-offset: 2 -*- */

package nwnintf.format.tlk;

import nwnintf.format.*;

import java.util.*;
import java.io.*;

/**
 * Represent a "tlk" file, particulary dialog.tlk
 */

public class TlkFile {

  // Byte offsets of elements in the header.
  public static int TYPE           = 0;
  public static int VERSION        = 4;
  public static int UNKNOWN1       = 8;
  public static int NSTRINGS       = 12;
  public static int STRDATAOFFSET  = 16;

  public static int FIRSTHEADER    = 20;
  public static int STRHED_TYPE    = 0;
  public static int STRHED_NAME    = 4;
  public static int STRHED_UNKNOWN1= 20;
  public static int STRHED_UNKNOWN2= 24;
  public static int STRHED_OFFSET  = 28;
  public static int STRHED_STRSIZE = 32;
  public static int STRHED_UNKNOWN3= 36;
  public static int STRHED_SIZE    = 40;

  RandomAccessFile raf_;

  int nstrings_;
  int strdataoffset_;

  public TlkFile() {
    raf_ = null;
  }

  public TlkFile( File f ) throws IOException {
    raf_ = new RandomAccessFile(f, "r");

    /* Cache a few values.
     */
    raf_.seek(NSTRINGS);
    nstrings_ = Prim.int32lsb( raf_ );

    raf_.seek(STRDATAOFFSET);
    strdataoffset_ = Prim.int32lsb( raf_ );
  }

  public String get(int idx) throws IOException {
    if( raf_ == null ) return Integer.toString(idx);

    raf_.seek( FIRSTHEADER + idx * STRHED_SIZE + STRHED_OFFSET );
    int offset = Prim.int32lsb( raf_ );
    raf_.seek( FIRSTHEADER + idx * STRHED_SIZE + STRHED_STRSIZE );
    int size = Prim.int32lsb( raf_ );

    raf_.seek( strdataoffset_ + offset );
    return Prim.string( raf_, size );
  }

}
