/* -*- c-basic-offset: 2 -*- */
package nwnintf.palette;

import java.util.*;
import java.io.File;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.io.IOException;
import java.awt.*;
import java.awt.event.*;

import nwnintf.format.*;
import nwnintf.format.mod.*;
import nwnintf.format.itp.*;
import nwnintf.format.itp.element.*;
import nwnintf.format.tlk.*;


public class MultiPalette extends JPanel {
    
  /* SinglePalettes keyed on e.g. "utc"
   *
   */
  Map palettemap_;
  /*
   * Keyed on 'utc' also.
   */
  Map buttonmap_;

  MultiPaletteTree mptree_;

  static Vector palettetypes_;

  SinglePalette activepalette_;

  CardLayout cardlayout_;
  JPanel paletteview_;

  ButtonGroup bgroup_;

  Collection selectionlisteners_;

  /**
   * @param mptree Represents the underlying palette data. Construct
   * one of these and re-use it for all palettes widgets created.
   * @param enabledtabs Contains strings like "utc", "uti" indicating
   * which resource types this palette should display.
   */
  public MultiPalette( MultiPaletteTree mptree, Collection enabledtabs ) {
    mptree_ = mptree;
    palettemap_ = new HashMap();
    buttonmap_  = new HashMap();
    selectionlisteners_ = new Vector();

    JPanel buttonview = new JPanel();
    paletteview_ = new JPanel();
    cardlayout_ = new CardLayout();
    paletteview_.setLayout( cardlayout_ );

    FlowLayout fl = new FlowLayout(); fl.setHgap(0); fl.setVgap(0);
    buttonview.setLayout( fl );

    /* Create SinglePalette objects.
     */
    for( Iterator it = palettetypes_.iterator(); it.hasNext(); ) {
      PaletteType pt = (PaletteType)it.next();
      SinglePalette widget =
	new SinglePalette( mptree_.treeByType(pt.restype()) );
      
      cardlayout_.addLayoutComponent( widget, pt.restype() );
      paletteview_.add( pt.restype(), widget );
      palettemap_.put( pt.restype(), widget );
    }

    /* Create buttons to switch between palettes.
     */
    bgroup_ = new ButtonGroup();
    JToggleButton firstb = null; /* The first button for a palette that is not
				  * disabled. */
    for( Iterator it = palettetypes_.iterator(); it.hasNext(); ) {
      PaletteType pt = (PaletteType)it.next();
      JToggleButton b;

      if( !new File(pt.iconpath()).canRead() ) {
	System.err.println("Can't read icon file " + pt.iconpath());
	b = new JToggleButton(pt.label());
      } else {
	ImageIcon icon = new ImageIcon( pt.iconpath() );
	b = new JToggleButton( icon );
	b.setPreferredSize( new Dimension( icon.getIconWidth() + 4,
					   icon.getIconHeight() + 2 ) );
      }

      b.setToolTipText(pt.label());
      b.setFocusPainted(false);

      buttonmap_.put( pt.restype(), b );
      if( enabledtabs == null || enabledtabs.contains( pt.restype() ) ) {
	b.addActionListener( new MultiButtonListener(this, pt.restype()) );
      } else {
	b.setEnabled(false);
      }
      buttonview.add(b);
      bgroup_.add(b);
      if( firstb == null && b.isEnabled() ) {
	firstb = b;
      }
    }

    setButtonBorders();

    /* Create dummy Standard | Custom buttons */
    Box dummybox = Box.createHorizontalBox();

    Insets insets = new Insets( 1,2,1,2 );
    ButtonGroup dummygroup = new ButtonGroup();
    JToggleButton b = new JToggleButton("Standard");
    b.setEnabled(false); b.setBorderPainted(false); b.setFocusPainted(false);
    b.setMargin(insets);
    dummygroup.add(b);
    dummybox.add(b);

    b = new JToggleButton("Custom");
    b.setFocusPainted(false); b.setMargin(insets);
    dummygroup.add(b);
    dummygroup.setSelected(b.getModel(),true);
    dummybox.add(b);
    dummybox.add(Box.createHorizontalGlue()); // force buttons to left side


    /* Put all the bits together.
     */
    setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
    add( buttonview );
    add( dummybox );
    add( paletteview_ );

    paletteview_.add("blank", new JPanel());
        
    cardlayout_.show(paletteview_,"blank");

    if( firstb != null ) firstb.doClick();

  }

  private void setButtonBorders() {
    for( Enumeration e = bgroup_.getElements(); e.hasMoreElements(); ) {
      JToggleButton b = (JToggleButton)e.nextElement();
      b.setBorderPainted( bgroup_.isSelected( b.getModel() ) );
    }
  }



  static Iterator paletteTypes() { return palettetypes_.iterator(); }

  private static void p( String itpfile, String iconpath,
			 String label, String restype ) {
    palettetypes_.add( new PaletteType(ResRef.parseFilename(itpfile),
				       iconpath,label,restype) );
  }
  static {
    palettetypes_ = new Vector();
    p("creaturepalcus.itp","nwnintf/icons/creature.gif","Creatures","utc" );
    p("doorpalcus.itp","nwnintf/icons/door.gif","Doors","utd" );
    p("encounterpalcus.itp","nwnintf/icons/encounter.gif","Encounters","ute" );
    p("itempalcus.itp","nwnintf/icons/item.gif","Items","uti" );
    p("placeablepalcus.itp","nwnintf/icons/placeable.gif","Placeable","utp" );
    p("soundpalcus.itp","nwnintf/icons/sound.gif","Sounds","uts" );
    p("storepalcus.itp","nwnintf/icons/merchant.gif","Merchants","utm" );
    p("triggerpalcus.itp","nwnintf/icons/trigger.gif","Triggers","utt" );
    p("waypointpalcus.itp","nwnintf/icons/waypoint.gif","Waypoints","utw" );
  }

  /**
   * Put selection highlight on a specific resource entry.
   */
  public void setSelection( ResRef resref ) {

    SinglePalette pal = (SinglePalette)palettemap_.get( resref.typeStr() );
    ((JToggleButton)buttonmap_.get( resref.typeStr() )).doClick();
    pal.setSelection( resref );
  }

  /**
   * Retrieve currently selected resref, if any. null if none.
   */
  public ResRef getSelection() {
    if( activepalette_ == null ) return null;
    return activepalette_.getSelection();
  }


  /**
   * The actionCommand of the ActionEvent sent to the listener will be
   * null, you must call back to getSelection() to find out what
   * is selected.
   */
  public void addSelectionListener( ActionListener listener ) {
    //jtree_.addTreeSelectionListener( new MultiPaletteSelectionThunk( this, listener ) );
    MultiPaletteSelectionThunk thunk = 
      new MultiPaletteSelectionThunk( this, listener );

    selectionlisteners_.add(listener);

    for( Iterator it = palettemap_.entrySet().iterator();
	 it.hasNext(); ) {
      Map.Entry ent = (Map.Entry)it.next();
      SinglePalette pal = (SinglePalette)ent.getValue();
      pal.addSelectionListener( thunk );
    }
  }

  public void showPane( String restype ) {
    SinglePalette pal = (SinglePalette)palettemap_.get(restype);
    activepalette_ = pal;
    cardlayout_.show( paletteview_, restype );
    setButtonBorders();

    /* Since we're looking at a different(maybe) pane the selection has
     * changed.
     */
    ActionEvent e = new ActionEvent(this, ActionEvent.ACTION_PERFORMED, null);
    for( Iterator it = selectionlisteners_.iterator(); it.hasNext(); ) {
      ((ActionListener)it.next()).actionPerformed(e);
    }
  }
}

/**
 * Receive ActionEvent from a single palette, transform into a new
 * ActionEvent with a more detailed command, and forward to an ActionListener.
 */
class MultiPaletteSelectionThunk implements ActionListener {
  MultiPalette pal_;
  ActionListener listener_;
  MultiPaletteSelectionThunk( MultiPalette pal, ActionListener listener ) {
    pal_ = pal;
    listener_ = listener;
  }

  public void actionPerformed( ActionEvent evt ) {
    ActionEvent aevt =
      new ActionEvent(pal_, ActionEvent.ACTION_PERFORMED, null);
    listener_.actionPerformed(aevt);
  }

}



class MultiButtonListener implements ActionListener {
  MultiPalette mpal_;
  String restype_;
  MultiButtonListener( MultiPalette mpal, String restype ) {
    mpal_ = mpal;
    restype_ = restype;
  }
  public void actionPerformed( ActionEvent evt ) {
    mpal_.showPane( restype_ );
  }
}

