/* -*- c-basic-offset: 2 -*- */
package nwnintf.palette;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;

import javax.swing.tree.*;
import nwnintf.format.*;
import nwnintf.format.mod.*;
import nwnintf.format.itp.*;
import nwnintf.format.itp.element.*;
import nwnintf.format.tlk.*;

/**
 * Wrap an itp palette file into a swing tree.
 */
public class PaletteTree  {

  DefaultMutableTreeNode treeroot_;
  ItpFile itp_;
  TlkFile tlk_;
  int restype_;

  Map resmap_; // Map resrefs to tree nodes.
  Map presmap_; // Map resrefs to PaletteNode

  /**
   * @param itp The itp file for the palette.
   * @param tlk Loaded tlk file.
   * @param restype Resource type referred to by this palette.
   *    e.g. creaturepalcus.itp refers to "utc".
   */
  public PaletteTree( ItpFile itp, TlkFile tlk, int restype )
			  throws java.io.IOException
  {
    itp_ = itp;
    tlk_ = tlk;
    restype_ = restype;
    resmap_ = new HashMap();
    presmap_ = new HashMap();
    treeroot_ = new DefaultMutableTreeNode();
    makenodes( treeroot_, (ItpListElt)(itp.root().element("MAIN")) );
  }

  /**
   * Retrieve a node by its resref.
   */
  public TreeNode nodeByResref( ResRef resref ) {
    return (TreeNode)(resmap_.get(new ResRef(resref.name(),restype_)));
  }


  private void makenodes( DefaultMutableTreeNode treenode,
			  ItpListElt entrylist ) 
			  throws java.io.IOException
  {

    for(int i = 0; i < entrylist.length(); ++i ) {

      ItpEntry ent = (ItpEntry)entrylist.entry(i);
      ItpElement name = ent.element("NAME");

      if( name == null ) {
	/* treenode does not describe a palette entry. It is either
	 * a non-terminal or an empty terminal.
	 */

	// Label at this node.
	int tlkidx = ((ItpUint32Elt)ent.element("STRREF")).value();

	PaletteNode pn = new PaletteNode( tlk_.get(tlkidx), restype_ );
	DefaultMutableTreeNode newnode = new DefaultMutableTreeNode( pn );
	treenode.add( newnode );

	// Recurse on children, if any.
	ItpElement children = ent.element("LIST");
	if( children != null ) {
	  // treenode has children.
	  makenodes( newnode, (ItpListElt)children );
	}

      } else {
	/* treenode describes a palette entry.
	 */
	PaletteNode pn = new PaletteNode( ent, ((ItpStringElt)name).value(),
					  restype_ );
	DefaultMutableTreeNode newnode = new DefaultMutableTreeNode( pn );
	resmap_.put( pn.resref(), newnode );
	presmap_.put( pn.resref(), pn );
	treenode.add( newnode );
      }

    }
  }

  TreeNode root() { return treeroot_; }

  /**
   * Set of Map.Entry key is ResRef. 
   * e.g. new ResRef("amauna","utc") value is a PaletteNode
   */
  public Set blueprintnodes() {
    return presmap_.entrySet();
  }
}

