/* -*- c-basic-offset: 2 -*- */
package nwnintf.palette;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.io.IOException;
import java.awt.*;
import java.awt.event.*;

import nwnintf.format.*;
import nwnintf.format.mod.*;
import nwnintf.format.itp.*;
import nwnintf.format.itp.element.*;
import nwnintf.format.tlk.*;


public class SinglePalette extends JPanel {
    
  PaletteTree ptree_;
  JTree jtree_;

  public SinglePalette( PaletteTree ptree ) {
    ptree_ = ptree;

    jtree_ = new JTree( ptree_.root() );
    jtree_.getSelectionModel().setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
    jtree_.setExpandsSelectedPaths(true);
    jtree_.setRootVisible(false);
    jtree_.setShowsRootHandles(true);

    DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
    renderer.setLeafIcon(null);
    renderer.setOpenIcon(null);
    renderer.setClosedIcon(null);
    jtree_.setCellRenderer(renderer);

    JScrollPane treeview = new JScrollPane( jtree_ );
    treeview.setMinimumSize( new Dimension(50,100) );
    treeview.setPreferredSize( new Dimension( 200, 400 ) );
    setLayout( new GridLayout(1,1) );
    add( treeview, BorderLayout.CENTER );
  }

  /**
   * Put selection highlight on a specific resource entry.
   */
  public void setSelection( ResRef resref ) {
    TreeNode node = ptree_.nodeByResref(resref);
    if( node != null ) {
      TreePath path = new TreePath( ((DefaultTreeModel)(jtree_.getModel())).getPathToRoot( node ) );
      jtree_.setSelectionPath( path );
    }
  }

  /**
   * Retrieve currently selected resref, if any.
   * @return null if none selected.
   */
  public ResRef getSelection() {
    Object node = jtree_.getLastSelectedPathComponent();
    if( node != null ) {
      Object pno = ((DefaultMutableTreeNode)node).getUserObject();
      return ((PaletteNode)pno).resref();
    }
    return null;
  }


  /**
   * The actionCommand of the ActionEvent sent to the listener will be
   * null. Must call getSelection() to learn what was selected.
   */
  public void addSelectionListener( ActionListener listener ) {
    jtree_.addTreeSelectionListener( new SinglePaletteSelectionThunk( this, listener ) );
  }

}

/**
 * Receive TreeSelectionEvent from the tree, transform into
 * ActionEvent, and forward to an ActionListener.
 */
class SinglePaletteSelectionThunk implements TreeSelectionListener {
  SinglePalette pal_;
  ActionListener listener_;
  SinglePaletteSelectionThunk( SinglePalette pal, ActionListener listener ) {
    pal_ = pal;
    listener_ = listener;
  }

  public void valueChanged( TreeSelectionEvent evt ) {

    ActionEvent aevt =
      new ActionEvent(this, ActionEvent.ACTION_PERFORMED,null);

    listener_.actionPerformed(aevt);
	  
  }

}
